/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#include <stdio.h>
#include <string.h>

#include "watchpoints.h"
#include "atemu.h"
#include "chip_data.h"
#include "chip_data_struct.h"

/* add a watchpoint */
/* possibly highlight the global variable being watched */
void add_watchpoint(int chipid, long addr)
{
    char hex[32];
    GtkTreeIter iter;

    gtk_list_store_append( chip_array[chipid].watchp, &iter );

    snprintf( hex, 32, "0x%08lx", addr );
    hex[31] = '\0';

    gtk_list_store_set( GTK_LIST_STORE(chip_array[chipid].watchp), &iter,
            0, hex, 1, addr, 2, 1, -1 );
    atemu_add_watch(chipid, addr);
}

void add_watchpoint_range(int chipid, long addr, int size, char *name)
{
    GtkTreeIter iter;
    long end_addr = addr + size;

    gtk_list_store_append( chip_array[chipid].watchp, &iter );

    gtk_list_store_set( GTK_LIST_STORE(chip_array[chipid].watchp), &iter,
            0, name, 1, addr, 2, size, -1 );
    
    for(; addr < end_addr; addr++){
        atemu_add_watch(chipid, addr);
    }
}

/* remove watchpoint */
void remove_watchpoint(int chipid, long addr)
{
    GtkTreeIter iter;
    long addrinnode;
    gboolean iterexists;
    int size;

    iterexists = gtk_tree_model_get_iter_first( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter );

    while( iterexists )
    {
        gtk_tree_model_get( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter,
                1, &addrinnode, 2, &size, -1 );
        if( addrinnode == addr )
        {
            if( size > 1 )
            {
                remove_watchpoint_range(chipid, addr, size);
                return;
            }
            gtk_list_store_remove( GTK_LIST_STORE(chip_array[chipid].watchp),
                    &iter );
            break;
        }
        iterexists = gtk_tree_model_iter_next( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter );
    }

    atemu_clear_watch(chipid, addr);
}

void remove_watchpoint_range(int chipid, long addr, int size)
{
    GtkTreeIter iter;
    long end_addr = addr + size;

    long addrinnode;
    gboolean iterexists;

    iterexists = gtk_tree_model_get_iter_first( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter );

    while( iterexists )
    {
        gtk_tree_model_get( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter,
                1, &addrinnode, -1 );
        if( addrinnode == addr )
        {
            gtk_list_store_remove( GTK_LIST_STORE(chip_array[chipid].watchp),
                    &iter );
            break;
        }
        iterexists = gtk_tree_model_iter_next( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter );
    }
    
    for(; addr < end_addr; addr++){
        atemu_clear_watch(chipid, addr);
    }
}
/*test if it is a watchpoint*/
int is_watchpoint(int chipid, long addr)
{
    GtkTreeIter iter;
    long addrinnode;
    gboolean iterexists;

    iterexists = gtk_tree_model_get_iter_first( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter );

    while( iterexists )
    {
        gtk_tree_model_get( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter,
                1, &addrinnode, -1 );
        if( addrinnode == addr )
        {
            return 1;
        }
        iterexists = gtk_tree_model_iter_next( GTK_TREE_MODEL(chip_array[chipid].watchp), &iter );
    }
    return 0;
}
